from enum import Enum
from typing import Optional, Union

import attr

from longship_api_client.models.chargepoint_dto_connectivity_status import (
    ChargepointDtoConnectivityStatus,
)
from longship_api_client.models.connector_operational_status_dto_operational_status import (
    ConnectorOperationalStatusDtoOperationalStatus,
)


class WebhookPayloadType(str, Enum):
    ChargePointBooted = "ChargePointBooted"
    OperationalStatusChanged = "OperationalStatusChanged"
    ConnectivityStatusChanged = "ConnectivityStatusChanged"
    SessionStart = "SessionStart"
    SessionUpdate = "SessionUpdate"
    SessionStop = "SessionStop"
    CDRCreated = "CdrCreated"
    LocationCreated = "LocationCreated"
    LocationUpdated = "LocationUpdated"
    MSPInvoiceProposalStatus = "MspInvoiceProposalStatus"
    Ping = "Ping"

    def __str__(self) -> str:
        return str(self.value)


class RegistrationStatusType(str, Enum):
    Accepted = "Accepted"
    Pending = "Pending"

    def __str__(self) -> str:
        return str(self.value)


@attr.s(auto_attribs=True)
class ChargePointBootedData:
    registrationstatus: RegistrationStatusType


@attr.s(auto_attribs=True)
class OperationalStatusChangedData:
    status: ConnectorOperationalStatusDtoOperationalStatus
    errorcode: str
    connectornumber: int
    statussource: str
    locationid: Optional[str] = attr.ib(default=None)
    evseid: Optional[str] = attr.ib(default=None)
    vendorid: Optional[str] = attr.ib(default=None)
    vendorerrorcode: Optional[str] = attr.ib(default=None)


@attr.s(auto_attribs=True)
class ConnectivityStatusChangedData:
    # TODO: Longship uses uppercase for the status, but the webhook response uses CamelCase
    status: ChargepointDtoConnectivityStatus


@attr.s(auto_attribs=True)
class SessionStartData:
    chargepointid: str
    connectornumber: int
    transactionid: str
    locationid: Optional[str] = attr.ib(default=None)
    evseid: Optional[str] = attr.ib(default=None)


@attr.s(auto_attribs=True)
class SessionUpdateData(SessionStartData):
    totalenergyinkwh: float
    totalduration: str
    totalcosts: float


@attr.s(auto_attribs=True)
class SessionStopData(SessionUpdateData):
    pass


@attr.s(auto_attribs=True)
class CDRCreatedData:
    chargepointid: str
    connectornumber: int
    totalenergyinkwh: float
    totalduration: str
    totalcosts: float
    transactionid: str
    locationid: Optional[str] = attr.ib(default=None)
    evseid: Optional[str] = attr.ib(default=None)


@attr.s(auto_attribs=True)
class LocationCreatedData:
    pass


@attr.s(auto_attribs=True)
class LocationUpdatedData:
    pass


@attr.s(auto_attribs=True)
class MSPInvoiceProposalStatusData:
    pass


@attr.s(auto_attribs=True)
class PingData:
    pass


@attr.s(auto_attribs=True)
class WebhookPayload:
    specversion: str
    id: str
    type: WebhookPayloadType
    subject: str
    time: str
    source: str
    datacontenttype: str
    data: Union[
        ChargePointBootedData,
        OperationalStatusChangedData,
        ConnectivityStatusChangedData,
        SessionStartData,
        SessionUpdateData,
        SessionStopData,
        CDRCreatedData,
        LocationCreatedData,
        LocationUpdatedData,
        MSPInvoiceProposalStatusData,
        PingData,
    ]

    def __attrs_post_init__(self):
        if self.type == WebhookPayloadType.ChargePointBooted:
            self.data = ChargePointBootedData(**self.data)
        elif self.type == WebhookPayloadType.OperationalStatusChanged:
            self.data = OperationalStatusChangedData(**self.data)
        elif self.type == WebhookPayloadType.ConnectivityStatusChanged:
            self.data["status"] = self.data["status"].upper()
            self.data = ConnectivityStatusChangedData(**self.data)
        elif self.type == WebhookPayloadType.SessionStart:
            self.data = SessionStartData(**self.data)
        elif self.type == WebhookPayloadType.SessionUpdate:
            self.data = SessionUpdateData(**self.data)
        elif self.type == WebhookPayloadType.SessionStop:
            self.data = SessionStopData(**self.data)
        elif self.type == WebhookPayloadType.CDRCreated:
            self.data = CDRCreatedData(**self.data)
        elif self.type == WebhookPayloadType.LocationCreated:
            self.data = LocationCreatedData(**self.data)
        elif self.type == WebhookPayloadType.LocationUpdated:
            self.data = LocationUpdatedData(**self.data)
        elif self.type == WebhookPayloadType.MSPInvoiceProposalStatus:
            self.data = MSPInvoiceProposalStatusData(**self.data)
        elif self.type == WebhookPayloadType.Ping:
            self.data = PingData(**self.data)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Union


T = TypeVar("T", bound="GetConfigurationRequest")


@_attrs_define
class GetConfigurationRequest:
    """
    Attributes:
        key (Union[Unset, List[str]]):
    """

    key: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        key: Union[Unset, List[str]] = UNSET
        if not isinstance(self.key, Unset):
            key = self.key

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if key is not UNSET:
            field_dict["key"] = key

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        key = cast(List[str], d.pop("key", UNSET))

        get_configuration_request = cls(
            key=key,
        )

        get_configuration_request.additional_properties = d
        return get_configuration_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.reimburse_started_by_token_dto_auth_method import (
    ReimburseStartedByTokenDtoAuthMethod,
)
from ..models.reimburse_started_by_token_dto_token_type import (
    ReimburseStartedByTokenDtoTokenType,
)


T = TypeVar("T", bound="ReimburseStartedByTokenDto")


@_attrs_define
class ReimburseStartedByTokenDto:
    """
    Attributes:
        uid (Union[Unset, str]):
        auth_reference (Union[Unset, str]):
        token_type (Union[Unset, ReimburseStartedByTokenDtoTokenType]):  Default:
            ReimburseStartedByTokenDtoTokenType.AD_HOC_USER.
        contract_id (Union[Unset, str]):
        auth_method (Union[Unset, ReimburseStartedByTokenDtoAuthMethod]):  Default:
            ReimburseStartedByTokenDtoAuthMethod.AUTH_REQUEST.
        provider_country_code (Union[Unset, str]):
        provider_party_id (Union[Unset, str]):
        token_ou_id (Union[Unset, str]):
        token_ou_name (Union[Unset, str]):
        token_ou (Union[Unset, str]):
    """

    uid: Union[Unset, str] = UNSET
    auth_reference: Union[Unset, str] = UNSET
    token_type: Union[Unset, ReimburseStartedByTokenDtoTokenType] = (
        ReimburseStartedByTokenDtoTokenType.AD_HOC_USER
    )
    contract_id: Union[Unset, str] = UNSET
    auth_method: Union[Unset, ReimburseStartedByTokenDtoAuthMethod] = (
        ReimburseStartedByTokenDtoAuthMethod.AUTH_REQUEST
    )
    provider_country_code: Union[Unset, str] = UNSET
    provider_party_id: Union[Unset, str] = UNSET
    token_ou_id: Union[Unset, str] = UNSET
    token_ou_name: Union[Unset, str] = UNSET
    token_ou: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        uid = self.uid

        auth_reference = self.auth_reference

        token_type: Union[Unset, str] = UNSET
        if not isinstance(self.token_type, Unset):
            token_type = self.token_type.value

        contract_id = self.contract_id

        auth_method: Union[Unset, str] = UNSET
        if not isinstance(self.auth_method, Unset):
            auth_method = self.auth_method.value

        provider_country_code = self.provider_country_code

        provider_party_id = self.provider_party_id

        token_ou_id = self.token_ou_id

        token_ou_name = self.token_ou_name

        token_ou = self.token_ou

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if uid is not UNSET:
            field_dict["uid"] = uid
        if auth_reference is not UNSET:
            field_dict["authReference"] = auth_reference
        if token_type is not UNSET:
            field_dict["tokenType"] = token_type
        if contract_id is not UNSET:
            field_dict["contractId"] = contract_id
        if auth_method is not UNSET:
            field_dict["authMethod"] = auth_method
        if provider_country_code is not UNSET:
            field_dict["providerCountryCode"] = provider_country_code
        if provider_party_id is not UNSET:
            field_dict["providerPartyId"] = provider_party_id
        if token_ou_id is not UNSET:
            field_dict["tokenOUId"] = token_ou_id
        if token_ou_name is not UNSET:
            field_dict["tokenOUName"] = token_ou_name
        if token_ou is not UNSET:
            field_dict["tokenOU"] = token_ou

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        uid = d.pop("uid", UNSET)

        auth_reference = d.pop("authReference", UNSET)

        _token_type = d.pop("tokenType", UNSET)
        token_type: Union[Unset, ReimburseStartedByTokenDtoTokenType]
        if isinstance(_token_type, Unset) or _token_type is None:
            token_type = UNSET
        else:
            token_type = ReimburseStartedByTokenDtoTokenType(_token_type)

        contract_id = d.pop("contractId", UNSET)

        _auth_method = d.pop("authMethod", UNSET)
        auth_method: Union[Unset, ReimburseStartedByTokenDtoAuthMethod]
        if isinstance(_auth_method, Unset) or _auth_method is None:
            auth_method = UNSET
        else:
            auth_method = ReimburseStartedByTokenDtoAuthMethod(_auth_method)

        provider_country_code = d.pop("providerCountryCode", UNSET)

        provider_party_id = d.pop("providerPartyId", UNSET)

        token_ou_id = d.pop("tokenOUId", UNSET)

        token_ou_name = d.pop("tokenOUName", UNSET)

        token_ou = d.pop("tokenOU", UNSET)

        reimburse_started_by_token_dto = cls(
            uid=uid,
            auth_reference=auth_reference,
            token_type=token_type,
            contract_id=contract_id,
            auth_method=auth_method,
            provider_country_code=provider_country_code,
            provider_party_id=provider_party_id,
            token_ou_id=token_ou_id,
            token_ou_name=token_ou_name,
            token_ou=token_ou,
        )

        reimburse_started_by_token_dto.additional_properties = d
        return reimburse_started_by_token_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="OrganizationUnitFinancialDetailsDto")


@_attrs_define
class OrganizationUnitFinancialDetailsDto:
    """
    Attributes:
        beneficiary_name (Union[Unset, str]):
        iban (Union[Unset, str]):
        bic (Union[Unset, str]):
    """

    beneficiary_name: Union[Unset, str] = UNSET
    iban: Union[Unset, str] = UNSET
    bic: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        beneficiary_name = self.beneficiary_name

        iban = self.iban

        bic = self.bic

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if beneficiary_name is not UNSET:
            field_dict["beneficiaryName"] = beneficiary_name
        if iban is not UNSET:
            field_dict["iban"] = iban
        if bic is not UNSET:
            field_dict["bic"] = bic

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        beneficiary_name = d.pop("beneficiaryName", UNSET)

        iban = d.pop("iban", UNSET)

        bic = d.pop("bic", UNSET)

        organization_unit_financial_details_dto = cls(
            beneficiary_name=beneficiary_name,
            iban=iban,
            bic=bic,
        )

        organization_unit_financial_details_dto.additional_properties = d
        return organization_unit_financial_details_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
from ..models.charging_meter_value_dto_measurand import ChargingMeterValueDtoMeasurand
import datetime
from ..models.charging_meter_value_dto_unit import ChargingMeterValueDtoUnit
from typing import Union


T = TypeVar("T", bound="ChargingMeterValueDto")


@_attrs_define
class ChargingMeterValueDto:
    """
    Attributes:
        timestamp (Union[Unset, datetime.datetime]):
        value (Union[Unset, str]):
        measurand (Union[Unset, ChargingMeterValueDtoMeasurand]):  Default:
            ChargingMeterValueDtoMeasurand.ENERGY_ACTIVE_EXPORT_REGISTER.
        unit (Union[Unset, ChargingMeterValueDtoUnit]):  Default: ChargingMeterValueDtoUnit.WH.
    """

    timestamp: Union[Unset, datetime.datetime] = UNSET
    value: Union[Unset, str] = UNSET
    measurand: Union[Unset, ChargingMeterValueDtoMeasurand] = (
        ChargingMeterValueDtoMeasurand.ENERGY_ACTIVE_EXPORT_REGISTER
    )
    unit: Union[Unset, ChargingMeterValueDtoUnit] = ChargingMeterValueDtoUnit.WH
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        timestamp: Union[Unset, str] = UNSET
        if not isinstance(self.timestamp, Unset):
            timestamp = self.timestamp.isoformat()

        value = self.value

        measurand: Union[Unset, str] = UNSET
        if not isinstance(self.measurand, Unset):
            measurand = self.measurand.value

        unit: Union[Unset, str] = UNSET
        if not isinstance(self.unit, Unset):
            unit = self.unit.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp
        if value is not UNSET:
            field_dict["value"] = value
        if measurand is not UNSET:
            field_dict["measurand"] = measurand
        if unit is not UNSET:
            field_dict["unit"] = unit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _timestamp = d.pop("timestamp", UNSET)
        timestamp: Union[Unset, datetime.datetime]
        if isinstance(_timestamp, Unset) or _timestamp is None:
            timestamp = UNSET
        else:
            timestamp = isoparse(_timestamp)

        value = d.pop("value", UNSET)

        _measurand = d.pop("measurand", UNSET)
        measurand: Union[Unset, ChargingMeterValueDtoMeasurand]
        if isinstance(_measurand, Unset) or _measurand is None:
            measurand = UNSET
        else:
            measurand = ChargingMeterValueDtoMeasurand(_measurand)

        _unit = d.pop("unit", UNSET)
        unit: Union[Unset, ChargingMeterValueDtoUnit]
        if isinstance(_unit, Unset) or _unit is None:
            unit = UNSET
        else:
            unit = ChargingMeterValueDtoUnit(_unit)

        charging_meter_value_dto = cls(
            timestamp=timestamp,
            value=value,
            measurand=measurand,
            unit=unit,
        )

        charging_meter_value_dto.additional_properties = d
        return charging_meter_value_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.authorization_charger_context_dto import (
        AuthorizationChargerContextDto,
    )
    from ..models.authorization_tenant_context_dto import AuthorizationTenantContextDto


T = TypeVar("T", bound="AuthorizationContextDetailsDto")


@_attrs_define
class AuthorizationContextDetailsDto:
    """
    Attributes:
        charger (Union[Unset, AuthorizationChargerContextDto]):
        tenant (Union[Unset, AuthorizationTenantContextDto]):
    """

    charger: Union[Unset, "AuthorizationChargerContextDto"] = UNSET
    tenant: Union[Unset, "AuthorizationTenantContextDto"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        charger: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.charger, Unset):
            charger = self.charger.to_dict()

        tenant: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tenant, Unset):
            tenant = self.tenant.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if charger is not UNSET:
            field_dict["charger"] = charger
        if tenant is not UNSET:
            field_dict["tenant"] = tenant

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.authorization_charger_context_dto import (
            AuthorizationChargerContextDto,
        )
        from ..models.authorization_tenant_context_dto import (
            AuthorizationTenantContextDto,
        )

        d = src_dict.copy()
        _charger = d.pop("charger", UNSET)
        charger: Union[Unset, AuthorizationChargerContextDto]
        if isinstance(_charger, Unset) or _charger is None:
            charger = UNSET
        else:
            charger = AuthorizationChargerContextDto.from_dict(_charger)

        _tenant = d.pop("tenant", UNSET)
        tenant: Union[Unset, AuthorizationTenantContextDto]
        if isinstance(_tenant, Unset) or _tenant is None:
            tenant = UNSET
        else:
            tenant = AuthorizationTenantContextDto.from_dict(_tenant)

        authorization_context_details_dto = cls(
            charger=charger,
            tenant=tenant,
        )

        authorization_context_details_dto.additional_properties = d
        return authorization_context_details_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.tariff_assertion_dto import TariffAssertionDto


T = TypeVar("T", bound="TariffInfoDto")


@_attrs_define
class TariffInfoDto:
    """
    Attributes:
        tariff_id (Union[Unset, str]):
        tariff_name (Union[Unset, str]):
        start_tariff (Union[Unset, float]):
        tariff_price (Union[Unset, float]):
        parking_tariff (Union[Unset, float]):
        parking_step_size_in_minutes (Union[Unset, int]):
        parking_grace_period_in_minutes (Union[Unset, int]):
        time_tariff (Union[Unset, float]):
        time_step_size_in_minutes (Union[Unset, int]):
        time_grace_period_in_minutes (Union[Unset, int]):
        assertions (Union[Unset, List['TariffAssertionDto']]):
    """

    tariff_id: Union[Unset, str] = UNSET
    tariff_name: Union[Unset, str] = UNSET
    start_tariff: Union[Unset, float] = UNSET
    tariff_price: Union[Unset, float] = UNSET
    parking_tariff: Union[Unset, float] = UNSET
    parking_step_size_in_minutes: Union[Unset, int] = UNSET
    parking_grace_period_in_minutes: Union[Unset, int] = UNSET
    time_tariff: Union[Unset, float] = UNSET
    time_step_size_in_minutes: Union[Unset, int] = UNSET
    time_grace_period_in_minutes: Union[Unset, int] = UNSET
    assertions: Union[Unset, List["TariffAssertionDto"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        tariff_id = self.tariff_id

        tariff_name = self.tariff_name

        start_tariff = self.start_tariff

        tariff_price = self.tariff_price

        parking_tariff = self.parking_tariff

        parking_step_size_in_minutes = self.parking_step_size_in_minutes

        parking_grace_period_in_minutes = self.parking_grace_period_in_minutes

        time_tariff = self.time_tariff

        time_step_size_in_minutes = self.time_step_size_in_minutes

        time_grace_period_in_minutes = self.time_grace_period_in_minutes

        assertions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.assertions, Unset):
            assertions = []
            for assertions_item_data in self.assertions:
                assertions_item = assertions_item_data.to_dict()
                assertions.append(assertions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if tariff_id is not UNSET:
            field_dict["tariffId"] = tariff_id
        if tariff_name is not UNSET:
            field_dict["tariffName"] = tariff_name
        if start_tariff is not UNSET:
            field_dict["startTariff"] = start_tariff
        if tariff_price is not UNSET:
            field_dict["tariffPrice"] = tariff_price
        if parking_tariff is not UNSET:
            field_dict["parkingTariff"] = parking_tariff
        if parking_step_size_in_minutes is not UNSET:
            field_dict["parkingStepSizeInMinutes"] = parking_step_size_in_minutes
        if parking_grace_period_in_minutes is not UNSET:
            field_dict["parkingGracePeriodInMinutes"] = parking_grace_period_in_minutes
        if time_tariff is not UNSET:
            field_dict["timeTariff"] = time_tariff
        if time_step_size_in_minutes is not UNSET:
            field_dict["timeStepSizeInMinutes"] = time_step_size_in_minutes
        if time_grace_period_in_minutes is not UNSET:
            field_dict["timeGracePeriodInMinutes"] = time_grace_period_in_minutes
        if assertions is not UNSET:
            field_dict["assertions"] = assertions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.tariff_assertion_dto import TariffAssertionDto

        d = src_dict.copy()
        tariff_id = d.pop("tariffId", UNSET)

        tariff_name = d.pop("tariffName", UNSET)

        start_tariff = d.pop("startTariff", UNSET)

        tariff_price = d.pop("tariffPrice", UNSET)

        parking_tariff = d.pop("parkingTariff", UNSET)

        parking_step_size_in_minutes = d.pop("parkingStepSizeInMinutes", UNSET)

        parking_grace_period_in_minutes = d.pop("parkingGracePeriodInMinutes", UNSET)

        time_tariff = d.pop("timeTariff", UNSET)

        time_step_size_in_minutes = d.pop("timeStepSizeInMinutes", UNSET)

        time_grace_period_in_minutes = d.pop("timeGracePeriodInMinutes", UNSET)

        assertions = []
        _assertions = d.pop("assertions", UNSET)
        for assertions_item_data in _assertions or []:
            assertions_item = TariffAssertionDto.from_dict(assertions_item_data)

            assertions.append(assertions_item)

        tariff_info_dto = cls(
            tariff_id=tariff_id,
            tariff_name=tariff_name,
            start_tariff=start_tariff,
            tariff_price=tariff_price,
            parking_tariff=parking_tariff,
            parking_step_size_in_minutes=parking_step_size_in_minutes,
            parking_grace_period_in_minutes=parking_grace_period_in_minutes,
            time_tariff=time_tariff,
            time_step_size_in_minutes=time_step_size_in_minutes,
            time_grace_period_in_minutes=time_grace_period_in_minutes,
            assertions=assertions,
        )

        tariff_info_dto.additional_properties = d
        return tariff_info_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class GetAllChargepointsOrderBy(str, Enum):
    MODEL = "model"
    NAME = "name"
    OUNAME = "ouname"
    SERIALNUMBER = "serialnumber"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class TariffAssertionDtoTariffType(str, Enum):
    DEFAULTTARIFF = "DefaultTariff"
    PRIVATETARIFF = "PrivateTariff"
    REIMBURSEMENTTARIFF = "ReimbursementTariff"
    SELLTARIFF = "SellTariff"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.tariff_assertion_dto_tariff_type import TariffAssertionDtoTariffType


T = TypeVar("T", bound="TariffAssertionDto")


@_attrs_define
class TariffAssertionDto:
    """
    Attributes:
        tariff_type (Union[Unset, TariffAssertionDtoTariffType]):  Default:
            TariffAssertionDtoTariffType.REIMBURSEMENTTARIFF.
        is_tariff_used (Union[Unset, bool]):
        tariff_result (Union[Unset, str]):
    """

    tariff_type: Union[Unset, TariffAssertionDtoTariffType] = (
        TariffAssertionDtoTariffType.REIMBURSEMENTTARIFF
    )
    is_tariff_used: Union[Unset, bool] = UNSET
    tariff_result: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        tariff_type: Union[Unset, str] = UNSET
        if not isinstance(self.tariff_type, Unset):
            tariff_type = self.tariff_type.value

        is_tariff_used = self.is_tariff_used

        tariff_result = self.tariff_result

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if tariff_type is not UNSET:
            field_dict["tariffType"] = tariff_type
        if is_tariff_used is not UNSET:
            field_dict["isTariffUsed"] = is_tariff_used
        if tariff_result is not UNSET:
            field_dict["tariffResult"] = tariff_result

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _tariff_type = d.pop("tariffType", UNSET)
        tariff_type: Union[Unset, TariffAssertionDtoTariffType]
        if isinstance(_tariff_type, Unset) or _tariff_type is None:
            tariff_type = UNSET
        else:
            tariff_type = TariffAssertionDtoTariffType(_tariff_type)

        is_tariff_used = d.pop("isTariffUsed", UNSET)

        tariff_result = d.pop("tariffResult", UNSET)

        tariff_assertion_dto = cls(
            tariff_type=tariff_type,
            is_tariff_used=is_tariff_used,
            tariff_result=tariff_result,
        )

        tariff_assertion_dto.additional_properties = d
        return tariff_assertion_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class GetAllChargepointsAccesstype(str, Enum):
    GUEST = "guest"
    NEW = "new"
    PRIVATE = "private"
    PUBLIC = "public"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="ReimbursementCustomerShareDto")


@_attrs_define
class ReimbursementCustomerShareDto:
    """
    Attributes:
        customer_share (Union[Unset, float]):
        energy_compensation (Union[Unset, float]):
        tenant_fee (Union[Unset, float]):
    """

    customer_share: Union[Unset, float] = UNSET
    energy_compensation: Union[Unset, float] = UNSET
    tenant_fee: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        customer_share = self.customer_share

        energy_compensation = self.energy_compensation

        tenant_fee = self.tenant_fee

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if customer_share is not UNSET:
            field_dict["customerShare"] = customer_share
        if energy_compensation is not UNSET:
            field_dict["energyCompensation"] = energy_compensation
        if tenant_fee is not UNSET:
            field_dict["tenantFee"] = tenant_fee

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        customer_share = d.pop("customerShare", UNSET)

        energy_compensation = d.pop("energyCompensation", UNSET)

        tenant_fee = d.pop("tenantFee", UNSET)

        reimbursement_customer_share_dto = cls(
            customer_share=customer_share,
            energy_compensation=energy_compensation,
            tenant_fee=tenant_fee,
        )

        reimbursement_customer_share_dto.additional_properties = d
        return reimbursement_customer_share_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class LocationPostDtoParkingType(str, Enum):
    ALONG_MOTORWAY = "ALONG_MOTORWAY"
    ON_DRIVEWAY = "ON_DRIVEWAY"
    ON_STREET = "ON_STREET"
    PARKING_GARAGE = "PARKING_GARAGE"
    PARKING_LOT = "PARKING_LOT"
    UNDERGROUND_GARAGE = "UNDERGROUND_GARAGE"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.private_emp_tariff_dto_power_type import PrivateEmpTariffDtoPowerType


T = TypeVar("T", bound="PrivateEmpTariffDto")


@_attrs_define
class PrivateEmpTariffDto:
    """
    Attributes:
        country_code (Union[Unset, str]):
        party_id (Union[Unset, str]):
        power_type (Union[Unset, PrivateEmpTariffDtoPowerType]):  Default: PrivateEmpTariffDtoPowerType.AC.
        use_public_tariff_when_kwh_is_cheaper (Union[Unset, bool]):
    """

    country_code: Union[Unset, str] = UNSET
    party_id: Union[Unset, str] = UNSET
    power_type: Union[Unset, PrivateEmpTariffDtoPowerType] = (
        PrivateEmpTariffDtoPowerType.AC
    )
    use_public_tariff_when_kwh_is_cheaper: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        country_code = self.country_code

        party_id = self.party_id

        power_type: Union[Unset, str] = UNSET
        if not isinstance(self.power_type, Unset):
            power_type = self.power_type.value

        use_public_tariff_when_kwh_is_cheaper = (
            self.use_public_tariff_when_kwh_is_cheaper
        )

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if country_code is not UNSET:
            field_dict["country_code"] = country_code
        if party_id is not UNSET:
            field_dict["party_id"] = party_id
        if power_type is not UNSET:
            field_dict["powerType"] = power_type
        if use_public_tariff_when_kwh_is_cheaper is not UNSET:
            field_dict["usePublicTariffWhenKwhIsCheaper"] = (
                use_public_tariff_when_kwh_is_cheaper
            )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        country_code = d.pop("country_code", UNSET)

        party_id = d.pop("party_id", UNSET)

        _power_type = d.pop("powerType", UNSET)
        power_type: Union[Unset, PrivateEmpTariffDtoPowerType]
        if isinstance(_power_type, Unset) or _power_type is None:
            power_type = UNSET
        else:
            power_type = PrivateEmpTariffDtoPowerType(_power_type)

        use_public_tariff_when_kwh_is_cheaper = d.pop(
            "usePublicTariffWhenKwhIsCheaper", UNSET
        )

        private_emp_tariff_dto = cls(
            country_code=country_code,
            party_id=party_id,
            power_type=power_type,
            use_public_tariff_when_kwh_is_cheaper=use_public_tariff_when_kwh_is_cheaper,
        )

        private_emp_tariff_dto.additional_properties = d
        return private_emp_tariff_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.energy_source_dto import EnergySourceDto
    from ..models.environmental_impact_dto import EnvironmentalImpactDto


T = TypeVar("T", bound="EnergyMixDto")


@_attrs_define
class EnergyMixDto:
    """
    Attributes:
        is_green_energy (Union[Unset, bool]):
        energy_sources (Union[Unset, List['EnergySourceDto']]):
        environ_impact (Union[Unset, List['EnvironmentalImpactDto']]):
    """

    is_green_energy: Union[Unset, bool] = UNSET
    energy_sources: Union[Unset, List["EnergySourceDto"]] = UNSET
    environ_impact: Union[Unset, List["EnvironmentalImpactDto"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        is_green_energy = self.is_green_energy

        energy_sources: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.energy_sources, Unset):
            energy_sources = []
            for energy_sources_item_data in self.energy_sources:
                energy_sources_item = energy_sources_item_data.to_dict()
                energy_sources.append(energy_sources_item)

        environ_impact: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.environ_impact, Unset):
            environ_impact = []
            for environ_impact_item_data in self.environ_impact:
                environ_impact_item = environ_impact_item_data.to_dict()
                environ_impact.append(environ_impact_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if is_green_energy is not UNSET:
            field_dict["is_green_energy"] = is_green_energy
        if energy_sources is not UNSET:
            field_dict["energy_sources"] = energy_sources
        if environ_impact is not UNSET:
            field_dict["environ_impact"] = environ_impact

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.energy_source_dto import EnergySourceDto
        from ..models.environmental_impact_dto import EnvironmentalImpactDto

        d = src_dict.copy()
        is_green_energy = d.pop("is_green_energy", UNSET)

        energy_sources = []
        _energy_sources = d.pop("energy_sources", UNSET)
        for energy_sources_item_data in _energy_sources or []:
            energy_sources_item = EnergySourceDto.from_dict(energy_sources_item_data)

            energy_sources.append(energy_sources_item)

        environ_impact = []
        _environ_impact = d.pop("environ_impact", UNSET)
        for environ_impact_item_data in _environ_impact or []:
            environ_impact_item = EnvironmentalImpactDto.from_dict(
                environ_impact_item_data
            )

            environ_impact.append(environ_impact_item)

        energy_mix_dto = cls(
            is_green_energy=is_green_energy,
            energy_sources=energy_sources,
            environ_impact=environ_impact,
        )

        energy_mix_dto.additional_properties = d
        return energy_mix_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.tariff_dto_tariff_type import TariffDtoTariffType
from dateutil.parser import isoparse
import datetime
from typing import Union
from ..models.tariff_dto_usage_type import TariffDtoUsageType

if TYPE_CHECKING:
    from ..models.tariff_price_dto import TariffPriceDto
    from ..models.private_emp_tariff_dto import PrivateEmpTariffDto
    from ..models.tariff_restriction import TariffRestriction


T = TypeVar("T", bound="TariffDto")


@_attrs_define
class TariffDto:
    """
    Attributes:
        tenant_id (str):
        id (Union[Unset, str]):
        ocpi_id (Union[Unset, str]):
        hubject_id (Union[Unset, str]):
        name (Union[Unset, str]):
        start_tariff (Union[Unset, float]):
        price (Union[Unset, float]):
        price_incl_vat (Union[Unset, float]):
        currency (Union[Unset, str]):
        last_updated (Union[Unset, datetime.datetime]):
        usage_type (Union[Unset, TariffDtoUsageType]):  Default: TariffDtoUsageType.SELL.
        tariff_type (Union[Unset, TariffDtoTariffType]):  Default: TariffDtoTariffType.AD_HOC_PAYMENT.
        vat (Union[Unset, float]):
        is_vat_relevant (Union[Unset, bool]):
        country_code (Union[Unset, str]):
        party_id (Union[Unset, str]):
        location_id (Union[Unset, str]):
        is_private_emp_tariff (Union[Unset, bool]):
        private_emp_tariff (Union[Unset, PrivateEmpTariffDto]):
        parking_tariff (Union[Unset, float]):
        parking_step_size_in_minutes (Union[Unset, int]):
        parking_grace_period_in_minutes (Union[Unset, int]):
        parking_tariff_restrictions (Union[Unset, List['TariffRestriction']]):
        time_tariff (Union[Unset, float]):
        time_step_size_in_minutes (Union[Unset, int]):
        time_grace_period_in_minutes (Union[Unset, int]):
        price_history (Union[Unset, List['TariffPriceDto']]):
        external_reference (Union[Unset, str]):
        deleted (Union[Unset, datetime.datetime]):
    """

    tenant_id: str
    id: Union[Unset, str] = UNSET
    ocpi_id: Union[Unset, str] = UNSET
    hubject_id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    start_tariff: Union[Unset, float] = UNSET
    price: Union[Unset, float] = UNSET
    price_incl_vat: Union[Unset, float] = UNSET
    currency: Union[Unset, str] = UNSET
    last_updated: Union[Unset, datetime.datetime] = UNSET
    usage_type: Union[Unset, TariffDtoUsageType] = TariffDtoUsageType.SELL
    tariff_type: Union[Unset, TariffDtoTariffType] = TariffDtoTariffType.AD_HOC_PAYMENT
    vat: Union[Unset, float] = UNSET
    is_vat_relevant: Union[Unset, bool] = UNSET
    country_code: Union[Unset, str] = UNSET
    party_id: Union[Unset, str] = UNSET
    location_id: Union[Unset, str] = UNSET
    is_private_emp_tariff: Union[Unset, bool] = UNSET
    private_emp_tariff: Union[Unset, "PrivateEmpTariffDto"] = UNSET
    parking_tariff: Union[Unset, float] = UNSET
    parking_step_size_in_minutes: Union[Unset, int] = UNSET
    parking_grace_period_in_minutes: Union[Unset, int] = UNSET
    parking_tariff_restrictions: Union[Unset, List["TariffRestriction"]] = UNSET
    time_tariff: Union[Unset, float] = UNSET
    time_step_size_in_minutes: Union[Unset, int] = UNSET
    time_grace_period_in_minutes: Union[Unset, int] = UNSET
    price_history: Union[Unset, List["TariffPriceDto"]] = UNSET
    external_reference: Union[Unset, str] = UNSET
    deleted: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        tenant_id = self.tenant_id

        id = self.id

        ocpi_id = self.ocpi_id

        hubject_id = self.hubject_id

        name = self.name

        start_tariff = self.start_tariff

        price = self.price

        price_incl_vat = self.price_incl_vat

        currency = self.currency

        last_updated: Union[Unset, str] = UNSET
        if not isinstance(self.last_updated, Unset):
            last_updated = self.last_updated.isoformat()

        usage_type: Union[Unset, str] = UNSET
        if not isinstance(self.usage_type, Unset):
            usage_type = self.usage_type.value

        tariff_type: Union[Unset, str] = UNSET
        if not isinstance(self.tariff_type, Unset):
            tariff_type = self.tariff_type.value

        vat = self.vat

        is_vat_relevant = self.is_vat_relevant

        country_code = self.country_code

        party_id = self.party_id

        location_id = self.location_id

        is_private_emp_tariff = self.is_private_emp_tariff

        private_emp_tariff: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.private_emp_tariff, Unset):
            private_emp_tariff = self.private_emp_tariff.to_dict()

        parking_tariff = self.parking_tariff

        parking_step_size_in_minutes = self.parking_step_size_in_minutes

        parking_grace_period_in_minutes = self.parking_grace_period_in_minutes

        parking_tariff_restrictions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.parking_tariff_restrictions, Unset):
            parking_tariff_restrictions = []
            for (
                parking_tariff_restrictions_item_data
            ) in self.parking_tariff_restrictions:
                parking_tariff_restrictions_item = (
                    parking_tariff_restrictions_item_data.to_dict()
                )
                parking_tariff_restrictions.append(parking_tariff_restrictions_item)

        time_tariff = self.time_tariff

        time_step_size_in_minutes = self.time_step_size_in_minutes

        time_grace_period_in_minutes = self.time_grace_period_in_minutes

        price_history: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.price_history, Unset):
            price_history = []
            for price_history_item_data in self.price_history:
                price_history_item = price_history_item_data.to_dict()
                price_history.append(price_history_item)

        external_reference = self.external_reference

        deleted: Union[Unset, str] = UNSET
        if not isinstance(self.deleted, Unset):
            deleted = self.deleted.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "tenantId": tenant_id,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if ocpi_id is not UNSET:
            field_dict["ocpiId"] = ocpi_id
        if hubject_id is not UNSET:
            field_dict["hubjectId"] = hubject_id
        if name is not UNSET:
            field_dict["name"] = name
        if start_tariff is not UNSET:
            field_dict["startTariff"] = start_tariff
        if price is not UNSET:
            field_dict["price"] = price
        if price_incl_vat is not UNSET:
            field_dict["priceInclVat"] = price_incl_vat
        if currency is not UNSET:
            field_dict["currency"] = currency
        if last_updated is not UNSET:
            field_dict["last_updated"] = last_updated
        if usage_type is not UNSET:
            field_dict["usageType"] = usage_type
        if tariff_type is not UNSET:
            field_dict["tariffType"] = tariff_type
        if vat is not UNSET:
            field_dict["vat"] = vat
        if is_vat_relevant is not UNSET:
            field_dict["isVatRelevant"] = is_vat_relevant
        if country_code is not UNSET:
            field_dict["country_code"] = country_code
        if party_id is not UNSET:
            field_dict["party_id"] = party_id
        if location_id is not UNSET:
            field_dict["locationId"] = location_id
        if is_private_emp_tariff is not UNSET:
            field_dict["isPrivateEmpTariff"] = is_private_emp_tariff
        if private_emp_tariff is not UNSET:
            field_dict["privateEmpTariff"] = private_emp_tariff
        if parking_tariff is not UNSET:
            field_dict["parkingTariff"] = parking_tariff
        if parking_step_size_in_minutes is not UNSET:
            field_dict["parkingStepSizeInMinutes"] = parking_step_size_in_minutes
        if parking_grace_period_in_minutes is not UNSET:
            field_dict["parkingGracePeriodInMinutes"] = parking_grace_period_in_minutes
        if parking_tariff_restrictions is not UNSET:
            field_dict["parkingTariffRestrictions"] = parking_tariff_restrictions
        if time_tariff is not UNSET:
            field_dict["timeTariff"] = time_tariff
        if time_step_size_in_minutes is not UNSET:
            field_dict["timeStepSizeInMinutes"] = time_step_size_in_minutes
        if time_grace_period_in_minutes is not UNSET:
            field_dict["timeGracePeriodInMinutes"] = time_grace_period_in_minutes
        if price_history is not UNSET:
            field_dict["priceHistory"] = price_history
        if external_reference is not UNSET:
            field_dict["externalReference"] = external_reference
        if deleted is not UNSET:
            field_dict["deleted"] = deleted

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.tariff_price_dto import TariffPriceDto
        from ..models.private_emp_tariff_dto import PrivateEmpTariffDto
        from ..models.tariff_restriction import TariffRestriction

        d = src_dict.copy()
        tenant_id = d.pop("tenantId")

        id = d.pop("id", UNSET)

        ocpi_id = d.pop("ocpiId", UNSET)

        hubject_id = d.pop("hubjectId", UNSET)

        name = d.pop("name", UNSET)

        start_tariff = d.pop("startTariff", UNSET)

        price = d.pop("price", UNSET)

        price_incl_vat = d.pop("priceInclVat", UNSET)

        currency = d.pop("currency", UNSET)

        _last_updated = d.pop("last_updated", UNSET)
        last_updated: Union[Unset, datetime.datetime]
        if isinstance(_last_updated, Unset) or _last_updated is None:
            last_updated = UNSET
        else:
            last_updated = isoparse(_last_updated)

        _usage_type = d.pop("usageType", UNSET)
        usage_type: Union[Unset, TariffDtoUsageType]
        if isinstance(_usage_type, Unset) or _usage_type is None:
            usage_type = UNSET
        else:
            usage_type = TariffDtoUsageType(_usage_type)

        _tariff_type = d.pop("tariffType", UNSET)
        tariff_type: Union[Unset, TariffDtoTariffType]
        if isinstance(_tariff_type, Unset) or _tariff_type is None:
            tariff_type = UNSET
        else:
            tariff_type = TariffDtoTariffType(_tariff_type)

        vat = d.pop("vat", UNSET)

        is_vat_relevant = d.pop("isVatRelevant", UNSET)

        country_code = d.pop("country_code", UNSET)

        party_id = d.pop("party_id", UNSET)

        location_id = d.pop("locationId", UNSET)

        is_private_emp_tariff = d.pop("isPrivateEmpTariff", UNSET)

        _private_emp_tariff = d.pop("privateEmpTariff", UNSET)
        private_emp_tariff: Union[Unset, PrivateEmpTariffDto]
        if isinstance(_private_emp_tariff, Unset) or _private_emp_tariff is None:
            private_emp_tariff = UNSET
        else:
            private_emp_tariff = PrivateEmpTariffDto.from_dict(_private_emp_tariff)

        parking_tariff = d.pop("parkingTariff", UNSET)

        parking_step_size_in_minutes = d.pop("parkingStepSizeInMinutes", UNSET)

        parking_grace_period_in_minutes = d.pop("parkingGracePeriodInMinutes", UNSET)

        parking_tariff_restrictions = []
        _parking_tariff_restrictions = d.pop("parkingTariffRestrictions", UNSET)
        for parking_tariff_restrictions_item_data in _parking_tariff_restrictions or []:
            parking_tariff_restrictions_item = TariffRestriction.from_dict(
                parking_tariff_restrictions_item_data
            )

            parking_tariff_restrictions.append(parking_tariff_restrictions_item)

        time_tariff = d.pop("timeTariff", UNSET)

        time_step_size_in_minutes = d.pop("timeStepSizeInMinutes", UNSET)

        time_grace_period_in_minutes = d.pop("timeGracePeriodInMinutes", UNSET)

        price_history = []
        _price_history = d.pop("priceHistory", UNSET)
        for price_history_item_data in _price_history or []:
            price_history_item = TariffPriceDto.from_dict(price_history_item_data)

            price_history.append(price_history_item)

        external_reference = d.pop("externalReference", UNSET)

        _deleted = d.pop("deleted", UNSET)
        deleted: Union[Unset, datetime.datetime]
        if isinstance(_deleted, Unset) or _deleted is None:
            deleted = UNSET
        else:
            deleted = isoparse(_deleted)

        tariff_dto = cls(
            tenant_id=tenant_id,
            id=id,
            ocpi_id=ocpi_id,
            hubject_id=hubject_id,
            name=name,
            start_tariff=start_tariff,
            price=price,
            price_incl_vat=price_incl_vat,
            currency=currency,
            last_updated=last_updated,
            usage_type=usage_type,
            tariff_type=tariff_type,
            vat=vat,
            is_vat_relevant=is_vat_relevant,
            country_code=country_code,
            party_id=party_id,
            location_id=location_id,
            is_private_emp_tariff=is_private_emp_tariff,
            private_emp_tariff=private_emp_tariff,
            parking_tariff=parking_tariff,
            parking_step_size_in_minutes=parking_step_size_in_minutes,
            parking_grace_period_in_minutes=parking_grace_period_in_minutes,
            parking_tariff_restrictions=parking_tariff_restrictions,
            time_tariff=time_tariff,
            time_step_size_in_minutes=time_step_size_in_minutes,
            time_grace_period_in_minutes=time_grace_period_in_minutes,
            price_history=price_history,
            external_reference=external_reference,
            deleted=deleted,
        )

        tariff_dto.additional_properties = d
        return tariff_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="RegularHoursDto")


@_attrs_define
class RegularHoursDto:
    """
    Attributes:
        weekday (Union[Unset, int]):
        period_begin (Union[Unset, str]):
        period_end (Union[Unset, str]):
    """

    weekday: Union[Unset, int] = UNSET
    period_begin: Union[Unset, str] = UNSET
    period_end: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        weekday = self.weekday

        period_begin = self.period_begin

        period_end = self.period_end

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if weekday is not UNSET:
            field_dict["weekday"] = weekday
        if period_begin is not UNSET:
            field_dict["period_begin"] = period_begin
        if period_end is not UNSET:
            field_dict["period_end"] = period_end

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        weekday = d.pop("weekday", UNSET)

        period_begin = d.pop("period_begin", UNSET)

        period_end = d.pop("period_end", UNSET)

        regular_hours_dto = cls(
            weekday=weekday,
            period_begin=period_begin,
            period_end=period_end,
        )

        regular_hours_dto.additional_properties = d
        return regular_hours_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.reimburse_info_dto_type import ReimburseInfoDtoType
from typing import Union

if TYPE_CHECKING:
    from ..models.location_tariff_distribution_dto import LocationTariffDistributionDto
    from ..models.reimbursement_tariff_dto import ReimbursementTariffDto
    from ..models.reimbursement_bank_details_dto import ReimbursementBankDetailsDto


T = TypeVar("T", bound="ReimburseInfoDto")


@_attrs_define
class ReimburseInfoDto:
    """
    Attributes:
        type (Union[Unset, ReimburseInfoDtoType]):  Default: ReimburseInfoDtoType.ORGANIZATIONUNIT.
        has_guest_usage (Union[Unset, bool]):
        has_guest_charging_reimbursement_fee (Union[Unset, bool]):
        country_code (Union[Unset, str]):
        party_id (Union[Unset, str]):
        external_organization_unit_id (Union[Unset, str]):
        external_organization_unit_name (Union[Unset, str]):
        external_organization_unit_reference (Union[Unset, str]):
        external_organization_unit_code (Union[Unset, str]):
        charge_card_emaid (Union[Unset, str]):
        charge_card_uid (Union[Unset, str]):
        charge_card_issuer (Union[Unset, str]):
        tariffs (Union[Unset, List['ReimbursementTariffDto']]):
        bank_details (Union[Unset, List['ReimbursementBankDetailsDto']]):
        tariff_distribution_id (Union[Unset, str]):
        tariff_distribution_history (Union[Unset, List['LocationTariffDistributionDto']]):
        ou (Union[Unset, str]):
        ou_id (Union[Unset, str]):
        ou_name (Union[Unset, str]):
    """

    type: Union[Unset, ReimburseInfoDtoType] = ReimburseInfoDtoType.ORGANIZATIONUNIT
    has_guest_usage: Union[Unset, bool] = UNSET
    has_guest_charging_reimbursement_fee: Union[Unset, bool] = UNSET
    country_code: Union[Unset, str] = UNSET
    party_id: Union[Unset, str] = UNSET
    external_organization_unit_id: Union[Unset, str] = UNSET
    external_organization_unit_name: Union[Unset, str] = UNSET
    external_organization_unit_reference: Union[Unset, str] = UNSET
    external_organization_unit_code: Union[Unset, str] = UNSET
    charge_card_emaid: Union[Unset, str] = UNSET
    charge_card_uid: Union[Unset, str] = UNSET
    charge_card_issuer: Union[Unset, str] = UNSET
    tariffs: Union[Unset, List["ReimbursementTariffDto"]] = UNSET
    bank_details: Union[Unset, List["ReimbursementBankDetailsDto"]] = UNSET
    tariff_distribution_id: Union[Unset, str] = UNSET
    tariff_distribution_history: Union[Unset, List["LocationTariffDistributionDto"]] = (
        UNSET
    )
    ou: Union[Unset, str] = UNSET
    ou_id: Union[Unset, str] = UNSET
    ou_name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        has_guest_usage = self.has_guest_usage

        has_guest_charging_reimbursement_fee = self.has_guest_charging_reimbursement_fee

        country_code = self.country_code

        party_id = self.party_id

        external_organization_unit_id = self.external_organization_unit_id

        external_organization_unit_name = self.external_organization_unit_name

        external_organization_unit_reference = self.external_organization_unit_reference

        external_organization_unit_code = self.external_organization_unit_code

        charge_card_emaid = self.charge_card_emaid

        charge_card_uid = self.charge_card_uid

        charge_card_issuer = self.charge_card_issuer

        tariffs: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.tariffs, Unset):
            tariffs = []
            for tariffs_item_data in self.tariffs:
                tariffs_item = tariffs_item_data.to_dict()
                tariffs.append(tariffs_item)

        bank_details: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.bank_details, Unset):
            bank_details = []
            for bank_details_item_data in self.bank_details:
                bank_details_item = bank_details_item_data.to_dict()
                bank_details.append(bank_details_item)

        tariff_distribution_id = self.tariff_distribution_id

        tariff_distribution_history: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.tariff_distribution_history, Unset):
            tariff_distribution_history = []
            for (
                tariff_distribution_history_item_data
            ) in self.tariff_distribution_history:
                tariff_distribution_history_item = (
                    tariff_distribution_history_item_data.to_dict()
                )
                tariff_distribution_history.append(tariff_distribution_history_item)

        ou = self.ou

        ou_id = self.ou_id

        ou_name = self.ou_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if has_guest_usage is not UNSET:
            field_dict["hasGuestUsage"] = has_guest_usage
        if has_guest_charging_reimbursement_fee is not UNSET:
            field_dict["hasGuestChargingReimbursementFee"] = (
                has_guest_charging_reimbursement_fee
            )
        if country_code is not UNSET:
            field_dict["countryCode"] = country_code
        if party_id is not UNSET:
            field_dict["partyId"] = party_id
        if external_organization_unit_id is not UNSET:
            field_dict["externalOrganizationUnitId"] = external_organization_unit_id
        if external_organization_unit_name is not UNSET:
            field_dict["externalOrganizationUnitName"] = external_organization_unit_name
        if external_organization_unit_reference is not UNSET:
            field_dict["externalOrganizationUnitReference"] = (
                external_organization_unit_reference
            )
        if external_organization_unit_code is not UNSET:
            field_dict["externalOrganizationUnitCode"] = external_organization_unit_code
        if charge_card_emaid is not UNSET:
            field_dict["chargeCardEMAID"] = charge_card_emaid
        if charge_card_uid is not UNSET:
            field_dict["chargeCardUID"] = charge_card_uid
        if charge_card_issuer is not UNSET:
            field_dict["chargeCardIssuer"] = charge_card_issuer
        if tariffs is not UNSET:
            field_dict["tariffs"] = tariffs
        if bank_details is not UNSET:
            field_dict["bankDetails"] = bank_details
        if tariff_distribution_id is not UNSET:
            field_dict["tariffDistributionId"] = tariff_distribution_id
        if tariff_distribution_history is not UNSET:
            field_dict["tariffDistributionHistory"] = tariff_distribution_history
        if ou is not UNSET:
            field_dict["ou"] = ou
        if ou_id is not UNSET:
            field_dict["ouId"] = ou_id
        if ou_name is not UNSET:
            field_dict["ouName"] = ou_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.location_tariff_distribution_dto import (
            LocationTariffDistributionDto,
        )
        from ..models.reimbursement_tariff_dto import ReimbursementTariffDto
        from ..models.reimbursement_bank_details_dto import ReimbursementBankDetailsDto

        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, ReimburseInfoDtoType]
        if isinstance(_type, Unset) or _type is None:
            type = UNSET
        else:
            type = ReimburseInfoDtoType(_type)

        has_guest_usage = d.pop("hasGuestUsage", UNSET)

        has_guest_charging_reimbursement_fee = d.pop(
            "hasGuestChargingReimbursementFee", UNSET
        )

        country_code = d.pop("countryCode", UNSET)

        party_id = d.pop("partyId", UNSET)

        external_organization_unit_id = d.pop("externalOrganizationUnitId", UNSET)

        external_organization_unit_name = d.pop("externalOrganizationUnitName", UNSET)

        external_organization_unit_reference = d.pop(
            "externalOrganizationUnitReference", UNSET
        )

        external_organization_unit_code = d.pop("externalOrganizationUnitCode", UNSET)

        charge_card_emaid = d.pop("chargeCardEMAID", UNSET)

        charge_card_uid = d.pop("chargeCardUID", UNSET)

        charge_card_issuer = d.pop("chargeCardIssuer", UNSET)

        tariffs = []
        _tariffs = d.pop("tariffs", UNSET)
        for tariffs_item_data in _tariffs or []:
            tariffs_item = ReimbursementTariffDto.from_dict(tariffs_item_data)

            tariffs.append(tariffs_item)

        bank_details = []
        _bank_details = d.pop("bankDetails", UNSET)
        for bank_details_item_data in _bank_details or []:
            bank_details_item = ReimbursementBankDetailsDto.from_dict(
                bank_details_item_data
            )

            bank_details.append(bank_details_item)

        tariff_distribution_id = d.pop("tariffDistributionId", UNSET)

        tariff_distribution_history = []
        _tariff_distribution_history = d.pop("tariffDistributionHistory", UNSET)
        for tariff_distribution_history_item_data in _tariff_distribution_history or []:
            tariff_distribution_history_item = LocationTariffDistributionDto.from_dict(
                tariff_distribution_history_item_data
            )

            tariff_distribution_history.append(tariff_distribution_history_item)

        ou = d.pop("ou", UNSET)

        ou_id = d.pop("ouId", UNSET)

        ou_name = d.pop("ouName", UNSET)

        reimburse_info_dto = cls(
            type=type,
            has_guest_usage=has_guest_usage,
            has_guest_charging_reimbursement_fee=has_guest_charging_reimbursement_fee,
            country_code=country_code,
            party_id=party_id,
            external_organization_unit_id=external_organization_unit_id,
            external_organization_unit_name=external_organization_unit_name,
            external_organization_unit_reference=external_organization_unit_reference,
            external_organization_unit_code=external_organization_unit_code,
            charge_card_emaid=charge_card_emaid,
            charge_card_uid=charge_card_uid,
            charge_card_issuer=charge_card_issuer,
            tariffs=tariffs,
            bank_details=bank_details,
            tariff_distribution_id=tariff_distribution_id,
            tariff_distribution_history=tariff_distribution_history,
            ou=ou,
            ou_id=ou_id,
            ou_name=ou_name,
        )

        reimburse_info_dto.additional_properties = d
        return reimburse_info_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.session_dto_approval_status import SessionDtoApprovalStatus
from dateutil.parser import isoparse
from ..models.session_dto_status import SessionDtoStatus
import datetime
from typing import Union
from ..models.session_dto_review_scenario_type import SessionDtoReviewScenarioType

if TYPE_CHECKING:
    from ..models.charging_meter_value_dto import ChargingMeterValueDto
    from ..models.session_location_dto import SessionLocationDto
    from ..models.price_info_dto import PriceInfoDto
    from ..models.tariff_info_dto import TariffInfoDto
    from ..models.session_thresholds_dto import SessionThresholdsDto
    from ..models.started_by_info_dto import StartedByInfoDto
    from ..models.charging_period_dto import ChargingPeriodDto


T = TypeVar("T", bound="SessionDto")


@_attrs_define
class SessionDto:
    """
    Attributes:
        id (Union[Unset, str]):
        tenant_id (Union[Unset, str]):
        charge_point_id (Union[Unset, str]):
        transaction_id (Union[Unset, int]):
        ocpp_transaction_id (Union[Unset, str]):
        connector_id (Union[Unset, int]):
        session_location (Union[Unset, SessionLocationDto]):
        id_tag (Union[Unset, str]):
        started_by_info (Union[Unset, StartedByInfoDto]):
        meter_start_in_wh (Union[Unset, int]):
        session_start (Union[Unset, datetime.datetime]):
        charging_periods (Union[Unset, List['ChargingPeriodDto']]):
        charging_meter_values (Union[Unset, List['ChargingMeterValueDto']]):
        meter_stop_in_wh (Union[Unset, int]):
        session_stop (Union[Unset, datetime.datetime]):
        status (Union[Unset, SessionDtoStatus]):  Default: SessionDtoStatus.ACTIVE.
        approval_status (Union[Unset, SessionDtoApprovalStatus]):  Default: SessionDtoApprovalStatus.VALUE_0.
        review_scenario_type (Union[Unset, SessionDtoReviewScenarioType]):  Default:
            SessionDtoReviewScenarioType.VALUE_0.
        total_energy_in_kwh (Union[Unset, float]):
        total_price (Union[Unset, float]):
        created (Union[Unset, datetime.datetime]):
        last_updated (Union[Unset, datetime.datetime]):
        ou (Union[Unset, str]):
        ou_id (Union[Unset, str]):
        ou_name (Union[Unset, str]):
        tariff_info (Union[Unset, TariffInfoDto]):
        price_info (Union[Unset, PriceInfoDto]):
        tariff_id (Union[Unset, str]):
        tariff_name (Union[Unset, str]):
        start_tariff (Union[Unset, float]):
        tariff_price (Union[Unset, float]):
        parking_tariff (Union[Unset, float]):
        thresholds (Union[Unset, SessionThresholdsDto]):
        parking_step_size (Union[Unset, int]):
        delay_in_minutes (Union[Unset, int]):
        parking_time_start (Union[Unset, datetime.datetime]):
    """

    id: Union[Unset, str] = UNSET
    tenant_id: Union[Unset, str] = UNSET
    charge_point_id: Union[Unset, str] = UNSET
    transaction_id: Union[Unset, int] = UNSET
    ocpp_transaction_id: Union[Unset, str] = UNSET
    connector_id: Union[Unset, int] = UNSET
    session_location: Union[Unset, "SessionLocationDto"] = UNSET
    id_tag: Union[Unset, str] = UNSET
    started_by_info: Union[Unset, "StartedByInfoDto"] = UNSET
    meter_start_in_wh: Union[Unset, int] = UNSET
    session_start: Union[Unset, datetime.datetime] = UNSET
    charging_periods: Union[Unset, List["ChargingPeriodDto"]] = UNSET
    charging_meter_values: Union[Unset, List["ChargingMeterValueDto"]] = UNSET
    meter_stop_in_wh: Union[Unset, int] = UNSET
    session_stop: Union[Unset, datetime.datetime] = UNSET
    status: Union[Unset, SessionDtoStatus] = SessionDtoStatus.ACTIVE
    approval_status: Union[Unset, SessionDtoApprovalStatus] = (
        SessionDtoApprovalStatus.VALUE_0
    )
    review_scenario_type: Union[Unset, SessionDtoReviewScenarioType] = (
        SessionDtoReviewScenarioType.VALUE_0
    )
    total_energy_in_kwh: Union[Unset, float] = UNSET
    total_price: Union[Unset, float] = UNSET
    created: Union[Unset, datetime.datetime] = UNSET
    last_updated: Union[Unset, datetime.datetime] = UNSET
    ou: Union[Unset, str] = UNSET
    ou_id: Union[Unset, str] = UNSET
    ou_name: Union[Unset, str] = UNSET
    tariff_info: Union[Unset, "TariffInfoDto"] = UNSET
    price_info: Union[Unset, "PriceInfoDto"] = UNSET
    tariff_id: Union[Unset, str] = UNSET
    tariff_name: Union[Unset, str] = UNSET
    start_tariff: Union[Unset, float] = UNSET
    tariff_price: Union[Unset, float] = UNSET
    parking_tariff: Union[Unset, float] = UNSET
    thresholds: Union[Unset, "SessionThresholdsDto"] = UNSET
    parking_step_size: Union[Unset, int] = UNSET
    delay_in_minutes: Union[Unset, int] = UNSET
    parking_time_start: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        tenant_id = self.tenant_id

        charge_point_id = self.charge_point_id

        transaction_id = self.transaction_id

        ocpp_transaction_id = self.ocpp_transaction_id

        connector_id = self.connector_id

        session_location: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.session_location, Unset):
            session_location = self.session_location.to_dict()

        id_tag = self.id_tag

        started_by_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.started_by_info, Unset):
            started_by_info = self.started_by_info.to_dict()

        meter_start_in_wh = self.meter_start_in_wh

        session_start: Union[Unset, str] = UNSET
        if not isinstance(self.session_start, Unset):
            session_start = self.session_start.isoformat()

        charging_periods: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.charging_periods, Unset):
            charging_periods = []
            for charging_periods_item_data in self.charging_periods:
                charging_periods_item = charging_periods_item_data.to_dict()
                charging_periods.append(charging_periods_item)

        charging_meter_values: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.charging_meter_values, Unset):
            charging_meter_values = []
            for charging_meter_values_item_data in self.charging_meter_values:
                charging_meter_values_item = charging_meter_values_item_data.to_dict()
                charging_meter_values.append(charging_meter_values_item)

        meter_stop_in_wh = self.meter_stop_in_wh

        session_stop: Union[Unset, str] = UNSET
        if not isinstance(self.session_stop, Unset):
            session_stop = self.session_stop.isoformat()

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        approval_status: Union[Unset, int] = UNSET
        if not isinstance(self.approval_status, Unset):
            approval_status = self.approval_status.value

        review_scenario_type: Union[Unset, int] = UNSET
        if not isinstance(self.review_scenario_type, Unset):
            review_scenario_type = self.review_scenario_type.value

        total_energy_in_kwh = self.total_energy_in_kwh

        total_price = self.total_price

        created: Union[Unset, str] = UNSET
        if not isinstance(self.created, Unset):
            created = self.created.isoformat()

        last_updated: Union[Unset, str] = UNSET
        if not isinstance(self.last_updated, Unset):
            last_updated = self.last_updated.isoformat()

        ou = self.ou

        ou_id = self.ou_id

        ou_name = self.ou_name

        tariff_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tariff_info, Unset):
            tariff_info = self.tariff_info.to_dict()

        price_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.price_info, Unset):
            price_info = self.price_info.to_dict()

        tariff_id = self.tariff_id

        tariff_name = self.tariff_name

        start_tariff = self.start_tariff

        tariff_price = self.tariff_price

        parking_tariff = self.parking_tariff

        thresholds: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.thresholds, Unset):
            thresholds = self.thresholds.to_dict()

        parking_step_size = self.parking_step_size

        delay_in_minutes = self.delay_in_minutes

        parking_time_start: Union[Unset, str] = UNSET
        if not isinstance(self.parking_time_start, Unset):
            parking_time_start = self.parking_time_start.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if tenant_id is not UNSET:
            field_dict["tenantId"] = tenant_id
        if charge_point_id is not UNSET:
            field_dict["chargePointId"] = charge_point_id
        if transaction_id is not UNSET:
            field_dict["transactionId"] = transaction_id
        if ocpp_transaction_id is not UNSET:
            field_dict["ocppTransactionId"] = ocpp_transaction_id
        if connector_id is not UNSET:
            field_dict["connectorId"] = connector_id
        if session_location is not UNSET:
            field_dict["sessionLocation"] = session_location
        if id_tag is not UNSET:
            field_dict["idTag"] = id_tag
        if started_by_info is not UNSET:
            field_dict["startedByInfo"] = started_by_info
        if meter_start_in_wh is not UNSET:
            field_dict["meterStartInWh"] = meter_start_in_wh
        if session_start is not UNSET:
            field_dict["sessionStart"] = session_start
        if charging_periods is not UNSET:
            field_dict["chargingPeriods"] = charging_periods
        if charging_meter_values is not UNSET:
            field_dict["chargingMeterValues"] = charging_meter_values
        if meter_stop_in_wh is not UNSET:
            field_dict["meterStopInWh"] = meter_stop_in_wh
        if session_stop is not UNSET:
            field_dict["sessionStop"] = session_stop
        if status is not UNSET:
            field_dict["status"] = status
        if approval_status is not UNSET:
            field_dict["approvalStatus"] = approval_status
        if review_scenario_type is not UNSET:
            field_dict["reviewScenarioType"] = review_scenario_type
        if total_energy_in_kwh is not UNSET:
            field_dict["totalEnergyInKwh"] = total_energy_in_kwh
        if total_price is not UNSET:
            field_dict["totalPrice"] = total_price
        if created is not UNSET:
            field_dict["created"] = created
        if last_updated is not UNSET:
            field_dict["lastUpdated"] = last_updated
        if ou is not UNSET:
            field_dict["ou"] = ou
        if ou_id is not UNSET:
            field_dict["ouId"] = ou_id
        if ou_name is not UNSET:
            field_dict["ouName"] = ou_name
        if tariff_info is not UNSET:
            field_dict["tariffInfo"] = tariff_info
        if price_info is not UNSET:
            field_dict["priceInfo"] = price_info
        if tariff_id is not UNSET:
            field_dict["tariffId"] = tariff_id
        if tariff_name is not UNSET:
            field_dict["tariffName"] = tariff_name
        if start_tariff is not UNSET:
            field_dict["startTariff"] = start_tariff
        if tariff_price is not UNSET:
            field_dict["tariffPrice"] = tariff_price
        if parking_tariff is not UNSET:
            field_dict["parkingTariff"] = parking_tariff
        if thresholds is not UNSET:
            field_dict["thresholds"] = thresholds
        if parking_step_size is not UNSET:
            field_dict["parkingStepSize"] = parking_step_size
        if delay_in_minutes is not UNSET:
            field_dict["delayInMinutes"] = delay_in_minutes
        if parking_time_start is not UNSET:
            field_dict["parkingTimeStart"] = parking_time_start

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.charging_meter_value_dto import ChargingMeterValueDto
        from ..models.session_location_dto import SessionLocationDto
        from ..models.price_info_dto import PriceInfoDto
        from ..models.tariff_info_dto import TariffInfoDto
        from ..models.session_thresholds_dto import SessionThresholdsDto
        from ..models.started_by_info_dto import StartedByInfoDto
        from ..models.charging_period_dto import ChargingPeriodDto

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        tenant_id = d.pop("tenantId", UNSET)

        charge_point_id = d.pop("chargePointId", UNSET)

        transaction_id = d.pop("transactionId", UNSET)

        ocpp_transaction_id = d.pop("ocppTransactionId", UNSET)

        connector_id = d.pop("connectorId", UNSET)

        _session_location = d.pop("sessionLocation", UNSET)
        session_location: Union[Unset, SessionLocationDto]
        if isinstance(_session_location, Unset) or _session_location is None:
            session_location = UNSET
        else:
            session_location = SessionLocationDto.from_dict(_session_location)

        id_tag = d.pop("idTag", UNSET)

        _started_by_info = d.pop("startedByInfo", UNSET)
        started_by_info: Union[Unset, StartedByInfoDto]
        if isinstance(_started_by_info, Unset) or _started_by_info is None:
            started_by_info = UNSET
        else:
            started_by_info = StartedByInfoDto.from_dict(_started_by_info)

        meter_start_in_wh = d.pop("meterStartInWh", UNSET)

        _session_start = d.pop("sessionStart", UNSET)
        session_start: Union[Unset, datetime.datetime]
        if isinstance(_session_start, Unset) or _session_start is None:
            session_start = UNSET
        else:
            session_start = isoparse(_session_start)

        charging_periods = []
        _charging_periods = d.pop("chargingPeriods", UNSET)
        for charging_periods_item_data in _charging_periods or []:
            charging_periods_item = ChargingPeriodDto.from_dict(
                charging_periods_item_data
            )

            charging_periods.append(charging_periods_item)

        charging_meter_values = []
        _charging_meter_values = d.pop("chargingMeterValues", UNSET)
        for charging_meter_values_item_data in _charging_meter_values or []:
            charging_meter_values_item = ChargingMeterValueDto.from_dict(
                charging_meter_values_item_data
            )

            charging_meter_values.append(charging_meter_values_item)

        meter_stop_in_wh = d.pop("meterStopInWh", UNSET)

        _session_stop = d.pop("sessionStop", UNSET)
        session_stop: Union[Unset, datetime.datetime]
        if isinstance(_session_stop, Unset) or _session_stop is None:
            session_stop = UNSET
        else:
            session_stop = isoparse(_session_stop)

        _status = d.pop("status", UNSET)
        status: Union[Unset, SessionDtoStatus]
        if isinstance(_status, Unset) or _status is None:
            status = UNSET
        else:
            status = SessionDtoStatus(_status)

        _approval_status = d.pop("approvalStatus", UNSET)
        approval_status: Union[Unset, SessionDtoApprovalStatus]
        if isinstance(_approval_status, Unset) or _approval_status is None:
            approval_status = UNSET
        else:
            approval_status = SessionDtoApprovalStatus(_approval_status)

        _review_scenario_type = d.pop("reviewScenarioType", UNSET)
        review_scenario_type: Union[Unset, SessionDtoReviewScenarioType]
        if isinstance(_review_scenario_type, Unset) or _review_scenario_type is None:
            review_scenario_type = UNSET
        else:
            review_scenario_type = SessionDtoReviewScenarioType(_review_scenario_type)

        total_energy_in_kwh = d.pop("totalEnergyInKwh", UNSET)

        total_price = d.pop("totalPrice", UNSET)

        _created = d.pop("created", UNSET)
        created: Union[Unset, datetime.datetime]
        if isinstance(_created, Unset) or _created is None:
            created = UNSET
        else:
            created = isoparse(_created)

        _last_updated = d.pop("lastUpdated", UNSET)
        last_updated: Union[Unset, datetime.datetime]
        if isinstance(_last_updated, Unset) or _last_updated is None:
            last_updated = UNSET
        else:
            last_updated = isoparse(_last_updated)

        ou = d.pop("ou", UNSET)

        ou_id = d.pop("ouId", UNSET)

        ou_name = d.pop("ouName", UNSET)

        _tariff_info = d.pop("tariffInfo", UNSET)
        tariff_info: Union[Unset, TariffInfoDto]
        if isinstance(_tariff_info, Unset) or _tariff_info is None:
            tariff_info = UNSET
        else:
            tariff_info = TariffInfoDto.from_dict(_tariff_info)

        _price_info = d.pop("priceInfo", UNSET)
        price_info: Union[Unset, PriceInfoDto]
        if isinstance(_price_info, Unset) or _price_info is None:
            price_info = UNSET
        else:
            price_info = PriceInfoDto.from_dict(_price_info)

        tariff_id = d.pop("tariffId", UNSET)

        tariff_name = d.pop("tariffName", UNSET)

        start_tariff = d.pop("startTariff", UNSET)

        tariff_price = d.pop("tariffPrice", UNSET)

        parking_tariff = d.pop("parkingTariff", UNSET)

        _thresholds = d.pop("thresholds", UNSET)
        thresholds: Union[Unset, SessionThresholdsDto]
        if isinstance(_thresholds, Unset) or _thresholds is None:
            thresholds = UNSET
        else:
            thresholds = SessionThresholdsDto.from_dict(_thresholds)

        parking_step_size = d.pop("parkingStepSize", UNSET)

        delay_in_minutes = d.pop("delayInMinutes", UNSET)

        _parking_time_start = d.pop("parkingTimeStart", UNSET)
        parking_time_start: Union[Unset, datetime.datetime]
        if isinstance(_parking_time_start, Unset) or _parking_time_start is None:
            parking_time_start = UNSET
        else:
            parking_time_start = isoparse(_parking_time_start)

        session_dto = cls(
            id=id,
            tenant_id=tenant_id,
            charge_point_id=charge_point_id,
            transaction_id=transaction_id,
            ocpp_transaction_id=ocpp_transaction_id,
            connector_id=connector_id,
            session_location=session_location,
            id_tag=id_tag,
            started_by_info=started_by_info,
            meter_start_in_wh=meter_start_in_wh,
            session_start=session_start,
            charging_periods=charging_periods,
            charging_meter_values=charging_meter_values,
            meter_stop_in_wh=meter_stop_in_wh,
            session_stop=session_stop,
            status=status,
            approval_status=approval_status,
            review_scenario_type=review_scenario_type,
            total_energy_in_kwh=total_energy_in_kwh,
            total_price=total_price,
            created=created,
            last_updated=last_updated,
            ou=ou,
            ou_id=ou_id,
            ou_name=ou_name,
            tariff_info=tariff_info,
            price_info=price_info,
            tariff_id=tariff_id,
            tariff_name=tariff_name,
            start_tariff=start_tariff,
            tariff_price=tariff_price,
            parking_tariff=parking_tariff,
            thresholds=thresholds,
            parking_step_size=parking_step_size,
            delay_in_minutes=delay_in_minutes,
            parking_time_start=parking_time_start,
        )

        session_dto.additional_properties = d
        return session_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TokenInfoDtoTokenType(str, Enum):
    ADHOCUSER = "AdHocUser"
    APPUSER = "AppUser"
    OTHER = "Other"
    RFID = "Rfid"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ChargePointAuthorizeGetDtoAuthorizationRequestType(str, Enum):
    AUTHORIZE = "Authorize"
    MANUAL = "Manual"
    STARTTRANSACTION = "StartTransaction"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class EnergySourceDtoSource(str, Enum):
    COAL = "COAL"
    GAS = "GAS"
    GENERAL_FOSSIL = "GENERAL_FOSSIL"
    GENERAL_GREEN = "GENERAL_GREEN"
    NUCLEAR = "NUCLEAR"
    SOLAR = "SOLAR"
    WATER = "WATER"
    WIND = "WIND"

    def __str__(self) -> str:
        return str(self.value)
